// MSN Messenger plug-in for VUPlayer
// Copyright (C) 2005 James Chapman
// http://www.vuplayer.com

// Original MSN Messenger source code from
// http://www.msnfanatic.com

#include <windows.h>
#include "vu_vis.h"

#define DELAY			1000
#define MAXSTRING		255
#define MAXFORMAT		64

#define MAGIC_NUMBER	0x547

COPYDATASTRUCT msndata;
const WCHAR DEFAULTFORMAT[]=L"{0} - {1}";

LRESULT CALLBACK WndProc(HWND hwnd, UINT message, WPARAM wParam, LPARAM lParam);
void SendMSNString();
void GetConfig();

char description[32]="MSN Messenger";
char wndclass[]="vuplayer_msn";
HINSTANCE hInstance;
HWND hWnd;
HWND hWndParent;
VU_INFO* pInfo;
UINT bUpdate=0;
DWORD bPlaying=0;
WCHAR wArtist[256];
WCHAR wTitle[256];
WCHAR wAlbum[256];
WCHAR prevArtist[256];
WCHAR prevTitle[256];
WCHAR prevAlbum[256];
WCHAR msnformat[MAXFORMAT];
WCHAR msnbuffer[MAXSTRING+1];

/////////////////////
// Exported functions
/////////////////////

LPCTSTR QueryVisual()
{
	return description;
}

HWND Init(VU_HEADER* pHdr)
{
	ZeroMemory(msnbuffer,sizeof(msnbuffer));
	ZeroMemory(wArtist,sizeof(wArtist));
	ZeroMemory(wTitle,sizeof(wTitle));
	ZeroMemory(wAlbum,sizeof(wAlbum));
	ZeroMemory(prevArtist,sizeof(prevArtist));
	ZeroMemory(prevTitle,sizeof(prevTitle));
	ZeroMemory(prevAlbum,sizeof(prevAlbum));
	GetConfig();
	hWndParent=pHdr->hWndParent;
	pInfo=pHdr->pInfo;
	hInstance=pHdr->hInstance;
	WNDCLASS wc;
	ZeroMemory(&wc,sizeof(WNDCLASS));
	wc.lpfnWndProc=WndProc;
	wc.hInstance=pHdr->hInstance;
	wc.lpszClassName=wndclass;
	if(RegisterClass(&wc))			// Register a class and create a hidden window for our vis
	{
		hWnd=CreateWindow(wndclass,description,0,0,0,32,32,0,0,pHdr->hInstance,0);
		if(hWnd)
			return hWnd;
		UnregisterClass(wndclass,hInstance);
	}
	return 0;
}

void Quit()
{
	bPlaying=0;
	SendMSNString();
	DestroyWindow(hWnd);
	UnregisterClass(wndclass,hInstance);
}

int Render()
{
	bUpdate=0;
	MultiByteToWideChar(GetACP(),MB_PRECOMPOSED,pInfo->lpstrArtist,-1,wArtist,256);
	MultiByteToWideChar(GetACP(),MB_PRECOMPOSED,pInfo->lpstrTitle,-1,wTitle,256);
	MultiByteToWideChar(GetACP(),MB_PRECOMPOSED,pInfo->lpstrAlbum,-1,wAlbum,256);
	if(bPlaying!=(pInfo->state&STATE_PLAYING))
	{
		bPlaying=pInfo->state&STATE_PLAYING;
		bUpdate=1;
	}
	if(wcscmp(wArtist,prevArtist)||wcscmp(wTitle,prevTitle)||wcscmp(wAlbum,prevAlbum))
	{
		wcscpy(prevArtist,wArtist);
		wcscpy(prevTitle,wTitle);
		wcscpy(prevAlbum,wAlbum);
		bUpdate=1;
	}
	if(bUpdate)
		SendMSNString();
	return DELAY;
}

void SendMSNString()
{
	HWND msnui=FindWindow("MsnMsgrUIManager",NULL);
	if(msnui)
	{
		wcscpy(msnbuffer,L"\\0Music\\0");
		if(bPlaying)
			wcscat(msnbuffer,L"1\\0");
		else
			wcscat(msnbuffer,L"0\\0");
		wcsncat(msnbuffer,msnformat,MAXSTRING-wcslen(msnbuffer));
		wcsncat(msnbuffer,L"\\0",MAXSTRING-wcslen(msnbuffer));
		wcsncat(msnbuffer,wArtist,MAXSTRING-wcslen(msnbuffer));
		wcsncat(msnbuffer,L"\\0",MAXSTRING-wcslen(msnbuffer));
		wcsncat(msnbuffer,wTitle,MAXSTRING-wcslen(msnbuffer));
		wcsncat(msnbuffer,L"\\0",MAXSTRING-wcslen(msnbuffer));
		wcsncat(msnbuffer,wAlbum,MAXSTRING-wcslen(msnbuffer));
		wcsncat(msnbuffer,L"\\0",MAXSTRING-wcslen(msnbuffer));
		msndata.dwData=MAGIC_NUMBER;
		msndata.lpData=&msnbuffer;
		msndata.cbData=(wcslen(msnbuffer)+1)*2;
		SendMessage(msnui,WM_COPYDATA,(WPARAM)hWndParent,(LPARAM)&msndata);
	}
}

void GetConfig()											// Get our config data from the registry
{
	BOOL createkey=0;
	HKEY hKey;
	char buffer[MAXFORMAT];
	DWORD bufSize=MAXFORMAT;
	if(RegOpenKeyEx(HKEY_CURRENT_USER,"Software\\VUPlayer\\Messenger",0,KEY_QUERY_VALUE,&hKey)==ERROR_SUCCESS)
	{
		if((RegQueryValueEx(hKey,"format",0,0,(BYTE*)buffer,&bufSize)==ERROR_SUCCESS)&&(bufSize>1))
		{
			if(!MultiByteToWideChar(GetACP(),MB_PRECOMPOSED,buffer,-1,msnformat,MAXFORMAT))
				createkey=1;
		}
		else
			createkey=1;
		RegCloseKey(hKey);
	}
	else
		createkey=1;
	if(createkey)
	{
		wcscpy(msnformat,DEFAULTFORMAT);
		DWORD disp;
		if(RegCreateKeyEx(HKEY_CURRENT_USER,"Software\\VUPlayer\\Messenger",0,0,REG_OPTION_NON_VOLATILE,KEY_ALL_ACCESS,0,&hKey,&disp)==ERROR_SUCCESS)
		{
			if(WideCharToMultiByte(GetACP(),0,msnformat,-1,buffer,MAXFORMAT,0,0))
				RegSetValueEx(hKey,"format",0,REG_SZ,(BYTE*)buffer,strlen(buffer)+1);
			RegCloseKey(hKey);
		}
	}
}

LRESULT CALLBACK WndProc(HWND hwnd, UINT message, WPARAM wParam, LPARAM lParam)
{
	switch(message)
	{
	case WM_CREATE:
		return 0;
	case WM_CLOSE:											// Signal to VUPlayer that we want to quit
		PostQuitMessage(0);
		return 0;
	}
	return DefWindowProc(hwnd,message,wParam,lParam);
}

